@@ -1,6 +1,6 @@
 import sqlalchemy as sa
 from sqlalchemy_utils.functions import create_database
-from pyg_base import cache, cfg_read, as_list, dictable, lower, last, loop, replace, Dict, is_dict, is_dictable, is_strs, is_str, is_int, is_date, dt2str, ulist, try_back, unique
+from pyg_base import cache, cfg_read, as_list, dictable, lower, last, loop, replace, Dict, is_dict, is_dictable, is_strs, is_str, is_int, is_date, dt2str, ulist, try_back, unique, is_primitive
 from pyg_encoders import as_reader, as_writer, dumps, loads
 from sqlalchemy import Table, Column, Integer, String, MetaData, Identity, Float, DATE, DATETIME, TIME, select, func, not_, desc, asc
 from sqlalchemy.orm import Session
@@ -30,6 +30,95 @@ _types = {str: String, 'str' : String,
 
 _orders = {1 : asc, True: asc, 'asc': asc, asc : asc, -1: desc, False: desc, 'desc': desc, desc: desc}
 
+def _relabel(res, selection, strict = True):
+    """
+    selection defines the CASE we want the columns to be in
+    We convert either columns or dicts into the case from 
+
+    Parameters
+    ----------
+    res : pd.DataFrame/dict
+        original value
+    selection : str/list of str
+        columnes/keys in the case we want them
+    strict : bool, optional
+        if False, will replace opportunistically. If True, needs all res & selection to match. The default is True.
+
+    Returns
+    -------
+    pd.DataFrame/dict
+        same object with case matching
+        
+    Example
+    -------
+    >>> res = dict(a = 1, b = 2, c = 3)
+    >>> selection  = ['A', 'B']
+    >>> assert _relabel(res, selection) == res                  ## no replacement, strict
+    >>> assert _relabel(res, selection, strict = False) == dict(A = 1, B = 2, c = 3) ## replaceme what is possible
+    >>> assert _relabel(res, ['A', 'B', 'C']) == dict(A = 1, B = 2, C = 3)
+    >>> assert _relabel(res, ['A', 'B', 'C'], True) == dict(A = 1, B = 2, C = 3)
+
+    """
+    if not is_strs(selection):
+        return res
+    lower_selection = lower(as_list(selection))
+    if isinstance(res, pd.DataFrame):
+        columns = list(res.columns)
+        if columns == selection:
+            return res
+        if lower(list(res.columns)) == lower_selection:
+            res.columns = selection
+        elif not strict:
+            lower2selection = dict(zip(lower_selection, selection))
+            res.columns = [lower2selection.get(col.lower(),col) for col in res.columns]    
+        return res
+    elif isinstance(res, dict):
+        if strict and sorted(lower(list(res.keys()))) == sorted(lower_selection):
+            columns = res.keys()
+            if sorted(columns) == sorted(selection): ## nothing to do
+                return res
+            lower2selection = dict(zip(lower_selection, selection))
+            return type(res)({lower2selection[key.lower()] : value for key, value in res.items()})
+        elif not strict:
+            lower2selection = dict(zip(lower_selection, selection))
+            return type(res)({lower2selection.get(key.lower(), key): value for key, value in res.items()})
+        else:
+            return res            
+    else:
+        return res       
+
+def _get_columns(table):
+    """
+    returns a dict from lower-case columns to actual columns
+    works with tables, join objects or sql_cursors
+    """
+    if isinstance(table, sql_cursor):
+        table = table._table
+    cols = table.columns
+    res = {}
+    for col in cols:
+        name = col.name.lower()
+        res[name] = res.get(name, []) + [col]
+    return res
+
+
+def _pw_filter(col, v):
+    if is_regex(v):
+        p = v.pattern
+        if not p.startswith('%'):
+            p = '%' + p
+        if not p.endswith('%'):
+            p = p + '%'
+        return col.like(p)
+    elif is_str(v) and v.startswith('%') and v.endswith('%'):
+        return col.like(v)
+    elif isinstance(v, list):
+        return sa.or_(*[_pw_filter(col, i) for i in v])
+    else:
+        return col == v
+    
+
+
 def pickle_loads(value):
     if isinstance(value, dict):
         return type(value)({k: pickle_loads(v) for k, v in value.items()})
@@ -40,23 +129,24 @@ def pickle_loads(value):
     else:
         return value
 
-def _pair_wise_filter(t, k, v):
-    """
-    t is a sqlalchemy column.c filter
-    """
-    if is_regex(v):
-        p = v.pattern
-        if not p.startswith('%'):
-            p = '%' + p
-        if not p.endswith('%'):
-            p = p + '%'
-        return t[k].like(p)
-    elif is_str(v) and v.startswith('%') and v.endswith('%'):
-        return t[k].like(v)
-    elif isinstance(v, list):
-        return sa.or_(*[_pair_wise_filter(t, k, i) for i in v])
-    else:
-        return t[k] == v
+# def _pair_wise_filter(t, k, v):
+#     """
+#     t is a sqlalchemy column.c filter
+#     """
+#     if is_regex(v):
+#         p = v.pattern
+#         if not p.startswith('%'):
+#             p = '%' + p
+#         if not p.endswith('%'):
+#             p = p + '%'
+#         return t[k].like(p)
+#     elif is_str(v) and v.startswith('%') and v.endswith('%'):
+#         return t[k].like(v)
+#     elif isinstance(v, list):
+#         return sa.or_(*[_pair_wise_filter(t, k, i) for i in v])
+#     else:
+#         return t[k] == v
+
 
 
 @cache
@@ -174,6 +264,7 @@ def get_cstr(*pairs, **connection):
         params = '&'.join('%s=%s'%(k,v) for k,v in connection.items())
         return 'mssql+pyodbc://%(server)s/%(db)s%(params)s'%dict(server=server, db = db, params = '?' +params if params else '')
 
+
 def create_schema(engine, schema):
     if schema is None:
         return
@@ -187,6 +278,7 @@ def create_schema(engine, schema):
             logger.info('creating schema: %s'%schema)
     return schema
 
+
 @cache
 def _get_engine(*pairs, **connection):    
     connection = _pairs2connection(*pairs, **connection)
@@ -205,6 +297,7 @@ def _get_engine(*pairs, **connection):
         e = sa.create_engine(cstr)       
     return e
 
+
 def get_engine(*pairs, **connection):
     """
     returns a sqlalchemy engine object
@@ -214,13 +307,19 @@ def get_engine(*pairs, **connection):
     return _get_engine(*pairs, **connection)
     
 
+
 @cache
 def _get_table(table_name, schema, db, server):
     e = _get_engine(server = server, db = db, schema = schema)
     meta = MetaData()
     return Table(table_name, meta, autoload_with = e, schema = schema)
 
-def sql_table(table, db = None, non_null = None, nullable = None, _id = None, schema = None, server = None, reader = None, writer = None, pk = None, doc = None, mode = None):
+
+
+
+def sql_table(table, db = None, non_null = None, nullable = None, _id = None, schema = None, 
+              server = None, reader = None, writer = None, pk = None, doc = None, mode = None, 
+              spec = None, selection = None, order = None, joint = None):
     """
     Creates a sql table. Can also be used to simply read table from the db
 
@@ -370,8 +469,9 @@ def sql_table(table, db = None, non_null = None, nullable = None, _id = None, sc
         tbl = Table(table_name, meta, *cols, schema = schema)
         meta.create_all(e)
     res = sql_cursor(table = tbl, schema = schema, db = db, server = server, engine = e, 
-                     spec = None, selection = None, reader = reader, writer = writer, 
-                     pk = list(pk) if isinstance(pk, dict) else pk, doc = doc)
+                     reader = reader, writer = writer, 
+                     pk = list(pk) if isinstance(pk, dict) else pk, doc = doc,
+                     spec = spec, selection = selection, order = order, joint = joint)
     return res
 
 class sql_cursor(object):
@@ -430,7 +530,34 @@ class sql_cursor(object):
     >>> assert t.distinct('surname') == ['gate', 'git']
     >>> assert t['surname'] == ['gate', 'git']
     >>> assert t[dict(name = 'yoav')] == t.inc(name = 'yoav')[0]
+    
+    
+    :Example: access is case-insensitive
+    ------------------------------------
+    >>> from pyg_sql import *
+    >>> t = sql_table(db = 'test', table = 'case_insensitive', nullable = dict(number = int, text = str))
+    >>> self = t.delete()
 
+    When we insert, we convert the case to the database columns convention:
+        
+    >>> assert t.insert_one(doc = dict(NUMBER = 1, TEXT = 'world')) == {'number': 1, 'text': 'world'}
+    >>> assert t.insert_one(dict(Number = 2, Text = 'hello')) == {'number': 2, 'text': 'hello'}
+
+    When we access the columns without specification, we get the database column names
+    
+    >>> assert t[0] == {'number': 1, 'text': 'world'}
+
+    When we filter or sort, we can filter using any case for column names
+    
+    >>> assert len(t.inc(NUMBER = 2)) == 1
+
+    assert t.sort('TEXT')[0] == {'number': 2, 'text': 'hello'}
+
+    When we SELECT, we can use any case, and key-case will follow your cursor.selection case:
+        
+    >>> assert t[['NUMBER','TexT']][0] == {'NUMBER': 1, 'TexT': 'world'}
+    >>> assert list(t[['Number','Text']].df().columns) == ['Number', 'Text']
+    >>> assert sorted(t[['Number','Text']][::].columns) == ['Number', 'Text']
 
     :Example: simple filtering
     --------------------------
@@ -497,7 +624,7 @@ class sql_cursor(object):
     a |m   |ann    |90   |a         |maths    |m         |andrews|mandy miles  
     a |z   |ann    |85   |a         |zoology  |z         |andrews|zoe zhenya   
     b |m   |ben    |70   |b         |maths    |m         |baxter |mandy miles  
-    b |g   |ben    |60   |b         |geography|g         |baxter |george graham
+    b |g   |ben    |60   |b         |geography|g         |baxter |pportgeorge graham
     c |g   |charles|50   |c         |geography|g         |cohen  |george graham
     c |z   |charles|45   |c         |zoology  |z         |cohen  |zoe zhenya       
 
@@ -767,31 +894,19 @@ class sql_cursor(object):
         cols = self.columns
         return dict(zip(lower(cols), cols))
 
-    def _joint_columns(self):
-        """
-        returns a mapping from lower col to both column name and table
-
-        """
-        rs = dictable(table = [j[0] for j in self.joint])
-        res = rs(grp = lambda table: dictable(column = table.columns, col = lower(table.columns))).ungroup().listby('col')
-        duplicates = dict(res.inc(lambda column: len(column)>1)['col','table'])
-        res = res.inc(lambda column: len(column)==1).do(last, 'column', 'table')
-        jcol = dict(res['col', 'column'])
-        jtbl = dict(res['col', 'table'])
-        return jcol, jtbl, duplicates
 
     def _kw(self, **kwargs):
-        columns = self._columns
-        jcol, jtbl, duplicates = self._joint_columns()
+        columns = _get_columns(self._table)
         conditions = []
         for k, v in kwargs.items():
             k = k.lower()
             if k in columns:
-                conditions.append(_pair_wise_filter(self.table.c, columns[k], v))
-            elif k in duplicates:
-                raise ValueError(f'column {k} exists in multiple tables: \n{duplicates[k]}')
-            elif k in jcol:
-                conditions.append(_pair_wise_filter(jtbl[k].c, jcol[k], v))
+                col = columns[k]
+                if len(col) > 1:
+                    raise ValueError(f'condition {k}={v} has two columns {col} by this name so cannot satisfy')
+                else:
+                    col = col[0]
+                conditions.append(_pw_filter(col, v))
             else:
                 raise ValueError(f'column {k} not found')
         return sa.and_(*conditions)
@@ -809,9 +924,6 @@ class sql_cursor(object):
         """
         if isinstance(expression, dict):
             return self._kw(**expression)
-            # expression = self._col(expression)
-            # t = self.table.c
-            # return sa.and_(*[_pair_wise_filter(t, k, v) for k,v in expression.items()]) 
         elif isinstance(expression, (list, tuple)):
             return sa.or_(*[self._c(v) for v in expression])            
         else:
@@ -821,37 +933,6 @@ class sql_cursor(object):
     def c(self):
         return self.table.c
     
-    def __ge__(self, kwargs):
-        """
-        provides a quick filtering by overloading operator:
-        
-        :Example:
-        ---------
-        >>> assert t >= dict(age = 30) == t.c.age >= 30
-        >>> assert t > dict(weight = 30, height = 20) == sa.and_(t.c.weight >= 30, t.c.height > 20)
-        """
-        if not isinstance(kwargs, dict) and len(self._ids) == 1:
-            kwargs = self._col({self._ids : kwargs})
-        c = self.table.c
-        return self.inc(sa.and_(*[c[key] >= value for key, value in kwargs.items()]))
-
-    def __gt__(self, kwargs):
-        if not isinstance(kwargs, dict) and len(self._ids) == 1:
-            kwargs = self._col({self._ids : kwargs})
-        c = self.table.c
-        return self.inc(sa.and_(*[c[key] > value for key, value in kwargs.items()]))
-
-    def __le__(self, kwargs):
-        if not isinstance(kwargs, dict) and len(self._ids) == 1:
-            kwargs = self._col({self._ids : kwargs})
-        c = self.table.c
-        return self.inc(sa.and_(*[c[key] <= value for key, value in kwargs.items()]))
-
-    def __lt__(self, kwargs):
-        if not isinstance(kwargs, dict) and len(self._ids) == 1:
-            kwargs = self._col({self._ids : kwargs})
-        c = self.table.c
-        return self.inc(sa.and_(*[c[key] < value for key, value in kwargs.items()]))
             
     @property
     def _pk(self):
@@ -906,6 +987,7 @@ class sql_cursor(object):
 
     inc = find
     where = find
+    filter = find
     
     def __sub__(self, other):
         """
@@ -927,7 +1009,11 @@ class sql_cursor(object):
             selection = [col for col in selection if col.name not in other]
             return self.select(selection)
             
-    def drop(self):
+    def drop(self, deleted = False):
+        if self.selection or self.spec:
+            raise ValueError('To avoid confusing .delete and .drop, dropping a table can only be done if there is no selection and no filtering')
+        if deleted:
+            self.deleted.drop()
         logger.info('dropping table: %s.%s.%s'%(_database(self.db), 
                                                   self.schema, 
                                                   self.table.name))
@@ -948,13 +1034,31 @@ class sql_cursor(object):
         res = self.table
         for j in as_list(self.joint):            
             right, onclose, isouter, full = j
+            if isinstance(right, partial):
+                right = right()
             if isinstance(right, sql_cursor):
                 right = right._table
             if is_strs(onclose):
                 onclose = as_list(onclose)
-                onclose = sa.and_(*[self.c[col] == right.c[col] for col in onclose])
-            elif is_dict(onclose):
-                onclose = sa.and_(*[self.c[key] == right.c[value] for key, value in onclose.items()])
+                onclose = dict(zip(onclose, onclose))
+            if is_dict(onclose):
+                lhs = _get_columns(self.table)
+                rhs = _get_columns(right)
+                conditions = []
+                for l,r in onclose.items():
+                    l = lhs[l.lower()]
+                    if len(l) > 1:
+                        raise ValueError(f'multiple columns of same name {l}')
+                    else:
+                        l = l[0]
+                    if is_str(r):
+                        r = rhs[r.lower()]
+                        if len(r) > 1:
+                            raise ValueError(f'multiple columns of same name {l}')
+                        else:
+                            r = r[0]
+                    conditions.append(l == r)
+                onclose = sa.and_(*conditions)
             res = res.join(right, onclose, isouter = isouter, full = full)
         return res
 
@@ -1019,7 +1123,7 @@ class sql_cursor(object):
         elif isinstance(column, dict):
             return type(column)({cols[k.lower()]: v for k, v in column.items()})
         else:
-            raise ValueError('column %s cannot be converted to table columns'%column)
+            raise ValueError(f'column {column} cannot be converted to table columns')
 
     @property    
     def columns(self):
@@ -1047,10 +1151,19 @@ class sql_cursor(object):
             for m in missing:
                 if m in doc:
                     missing[m].append(doc[m])
-        found = {k : v[0] for k, v in missing.items() if len(set(v)) == 1}
-        conflicted = {k : v for k, v in missing.items() if len(set(v)) > 1}
+        found = {}
+        conflicted = {} #
+        for k, v in missing.items():
+            if len(v) == 1:
+                found[k] = v[0]
+            elif len(v) > 1:
+                p = [i for i in v if is_primitive(i)]
+                if len(set(p)) == 1:
+                    found[k] = p[0]
+                else:
+                    conflicted[k] = v
         if conflicted:
-            raise ValueError('got multiple possible values for each of these columns: %s'%conflicted)
+            raise ValueError(f'got multiple possible values for each of these columns: {conflicted}')
         res.update(found)
         return res
                 
@@ -1066,12 +1179,13 @@ class sql_cursor(object):
             Suppose you have a document with EXTRA keys. Rather than filter the document, set ignore_bad_keys = True and we will drop irrelevant keys for you
 
         """
+        doc = _relabel(res = doc, selection = self.columns, strict = False) ## we want to replace what is possible since 
         edoc = self._dock(doc) if write else doc
         columns = self.columns
         if not ignore_bad_keys:
             bad_keys = {key: value for key, value in edoc.items() if key not in columns}
             if len(bad_keys) > 0:
-                raise ValueError('cannot insert into db a document with these keys: %s. The table only has these keys: %s'%(bad_keys, columns))        
+                raise ValueError(f'cannot insert into db a document with these keys: {bad_keys}. The table only has these keys: {columns}')        
         res = self._write_doc(edoc, columns = columns) if write else edoc
         if self._pk and not self._is_deleted():
             doc_id = self._id(res)
@@ -1163,7 +1277,7 @@ class sql_cursor(object):
             return Dict(doc) if type(doc) == dict else doc
         res = doc[self.doc]
         columns = columns or self.columns
-        for col in columns - self.doc:
+        for col in ulist(columns) - self.doc:
             if col in doc and col not in res:
                 res[col] = doc[col]
         return Dict(res) if type(res) == dict else res
@@ -1266,7 +1380,7 @@ class sql_cursor(object):
                 t = self.table.columns[col].type
                 if isinstance(t, NUMERIC) and not isinstance(t, (FLOAT, INT)):
                     res[col] = res[col].astype(float)
-        return res
+        return _relabel(res, self.selection)
             
     
     def _rows_to_docs(self, data, reader = None, load = True, columns = None):
@@ -1301,7 +1415,7 @@ class sql_cursor(object):
             row = data[0]
         doc = self._rows_to_docs(data = row, reader = reader, columns = columns)
         rtn = self._undock(doc)
-        return rtn
+        return _relabel(rtn, self.selection)
 
                 
     def __getitem__(self, value):
@@ -1311,8 +1425,18 @@ class sql_cursor(object):
         - t['key'] or t['key1', 'key2'] are equivalent to t.distinct('key') or t.distinct('key1', 'key2') 
         - t[['key1', 'key2']] will add a "SELECT key1, key2" to the statent
         - t[0] to grab a specific record. Note: this works better if you SORT the table first!, use t.sort('age')[10] to grab the name of the 11th youngest child in the class
-        - t[::] a slice of the data: t.sort('age')[:10] are the 10 youngest students     
+        - t[::] a slice of the data: t.sort('age')[:10] are the 10 youngest students 
         
+        :Example: support for columns case matching selection
+        ---------
+        >>> from pyg import * 
+        >>> t = sql_table(db = 'db', table = 't', nullable = ['a', 'b'])
+        >>> t.delete()
+        >>> t.insert(dict(a = dict(x=1,y=2), b = 2))
+        >>> assert t[['A']][::] == dictable(A = '1')
+        >>> assert t[['A']].df().columns[0] == 'A'
+        >>> assert t[['A']][0] == Dict(A = '1')
+        >>> assert t[['A', 'b']].read(0) == Dict(A = '1', b = '2')
         """
         if isinstance(value, list):
             return self.select(value)
@@ -1326,9 +1450,9 @@ class sql_cursor(object):
             if n == 1:
                 return res[0]
             elif n == 0:
-                raise ValueError('no records found for %s'%value)
+                raise ValueError(f'no records found for {value}')
             else:
-                raise ValueError('multiple %s records found for %s'%(n, value))
+                raise ValueError(f'multiple {n} records found for {value}')
 
         elif isinstance(value, slice):
             start, stop, step = value.start, value.stop, value.step
@@ -1341,7 +1465,7 @@ class sql_cursor(object):
             docs = self._rows_to_docs(reader = reader, **read)
             columns = read['columns'] #self.columns
             res = dictable([self._undock(doc, columns = columns) for doc in docs])
-            return res
+            return _relabel(res, self.selection)
 
         elif is_int(value):
             return self.read(value)
@@ -1354,12 +1478,13 @@ class sql_cursor(object):
         n = len(existing)
         if n == 0:
             if upsert is False:
-                raise ValueError('no documents found to update %s'%doc)
+                raise ValueError(f'no documents found to update {doc}')
             else:
                 return self.insert_one(doc)
         elif self._pk:
             return self.insert_one(doc)
         elif n == 1:
+            doc = _relabel(doc, self.columns, strict = False)
             edoc = self._dock(doc)
             wdoc = self._write_doc(edoc)
             for i in self._ids:
@@ -1370,7 +1495,7 @@ class sql_cursor(object):
             res.update(edoc)
             return self._undock(res)
         elif n > 1:
-            raise ValueError('multiple documents found matching %s '%doc)
+            raise ValueError(f'multiple documents found matching {doc}')
                 
             
     def insert_many(self, docs, write = True):
@@ -1382,6 +1507,7 @@ class sql_cursor(object):
         docs : list of dicts, dictable, pd.DataFrame
         """
         rs = dictable(docs)
+        rs = _relabel(rs, self.columns, strict = False)
         if len(rs) > 0:
             if self._pk and not self._is_deleted():
                 _ = [self.insert_one(doc, write = write) for doc in rs]
@@ -1456,22 +1582,49 @@ class sql_cursor(object):
             raise ValueError('no document found for %s %s %s'%(doc, args, kwargs))
         elif len(res) > 1:
             raise ValueError('multiple documents found for %s %s %s'%(doc, args, kwargs))
-                
+    
+    
     def _select(self):
         """
         performs a selection based on self.selection
+
+        >>> from pyg import *        
+        >>> t1  = sql_table(table = 't1', nullable = ['a', 'b', 'c'], db = 'db')
+        >>> t1.insert(dictable(a = 'a', b = ['a','b','c'], c = 'hi'))
+        t1[['a','b']].inc(B = 'b')
+
+        >>> t2  = sql_table(table = 't2', nullable = ['x', 'y', 'z'], db = 'db')
+        >>> t2.insert(dictable(x = 'x', y = ['a','b','c'], z = 'world'))
+
+        >>> t4  = sql_table(table = 't4', nullable = ['a', 'b', 'c'], db = 'db')
+        >>> t4.insert(dictable(a = 'u', b = ['a','b','c'], c = 'beautiful'))
+
+        t1.join(t2, t1.c['b']==t2.c['y'])[['a', 'b', 'z']][::]
+        t1.join(t4, 'b').inc(a = 'a')
+        [col.name for col in t1.table.join(t4.table, t1.c.b==t4.c.b).columns]
         """
         table = self._table
         if self.selection is None:
-            statement = select(table)
-        elif is_strs(self.selection):               
-            c = self.table.c
-            selection = self._col(as_list(self.selection))
-            selection = [c[v] for v in selection]
-            statement = select(selection).select_from(table)
-        else: ## user provided sql alchemy selection object
-            statement = select(self.selection).select_from(table)
+            return select(table)
+        columns = _get_columns(self._table)
+        selection = []
+        for col in as_list(self.selection):
+            if is_str(col):
+                col = col.lower()
+                if col in columns:
+                    s = columns[col]
+                    if len(s) > 1:
+                        raise ValueError(f'{col} is in two tables {s} so cannot resolve')
+                    else:
+                        s = s[0]                        
+                    selection.append(s)
+                else:
+                    raise ValueError(f'{col} not found in the table')
+            else:
+                selection.append(col) ## it is assumed to be a sqlalchemy selection
+        statement = select(selection).select_from(table)
         return statement
+
     
     def statement(self):
         """
@@ -1486,10 +1639,17 @@ class sql_cursor(object):
             statement = statement.where(self.spec)
         if self.order is not None:
             order = self.order
-            cols = self.table.columns
+            cols = _get_columns(self._table)
             if isinstance(order, (str,list)):
-                order = {o: 1 for o in as_list(order)}           
-            statement = statement.order_by(*[_orders[v](cols[k]) for k, v in order.items()])
+                order = {o: 1 for o in as_list(order)}
+            order_by = []
+            for k, v in order.items():
+                col = cols[k.lower()]
+                if len(col) > 1:
+                    raise ValueError(f'cannot sort as multiple col {k} in join: {col}')
+                else:
+                    order_by.append(_orders[v](col[0]))
+            statement = statement.order_by(*order_by)
         return statement
     
     def update(self, **kwargs):
@@ -1535,7 +1695,8 @@ class sql_cursor(object):
             res.full_delete()
         return self
 
-    def sort(self, order = None):
+    def sort(self, *order):
+        order = as_list(order)
         if not order:
             return self
         else:
